/* Welcome2L - Welcome to Linux, ANSI boot logo
 * Copyright (c)1998-1999 Jean-Marc Jacquet <jm@littleigloo.org>  
 * Little Igloo Org  http://www.LittleIgloo.org
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/utsname.h>
#include <sys/stat.h>
#include <locale.h>
#include <time.h>
#include <sys/time.h>
#include <ctype.h>
#include <argp.h>

#include "version.h"
#include "ansi.h"
#include "ansi/welcome1.h"
#include "ansi/welcome2.h"
#include "ansi/xmas.h"


#ifndef FALSE
#define FALSE (0)
#endif
#ifndef TRUE
#define TRUE (!FALSE)
#endif

#define NODENAME uname_buf.nodename
#define MACHINE uname_buf.machine


static struct utsname uname_buf;
static char *MEMORY;
static char TIME[30];
static char *WMESG;
static char *CPUID;
static char *Mhz;
static char want_getty_infos = FALSE;
static char THIS_SCREEN = 1;
static char last_color = FALSE;
static char WANT_NEWLINE = TRUE;
static int NEW_YEAR = FALSE;
static char XMAS_AUTO = FALSE;
static char IS_WELCOME = TRUE;
char XMAS_SCREEN = 3;
#define MAX_RAND_SCREEN 2


static void
draw_ansi_screen (unsigned char *ansi_screen, int ansi_screen_length, int ansi_screen_width)
{
  register int f;
  int this_width = ansi_screen_width << 1;

  for (f = 0; f < ansi_screen_length; f += 2)
    {
      if (f)
	if ((f % this_width) == 0)
	  ANSI_do_char ('\n');
      ANSI_do_color (ansi_screen[f + 1]);
      ANSI_do_char (ansi_screen[f]);
    }
}



static void
draw_welcome_screen (unsigned char *screen, int screen_length, int screen_width,
		     char color1, char color2, char color3, char color4,
		     char xnode, char ynode, char xtime, char ytime,
		     char xtty, char ytty,
		     char xwmsg, char ywmsg, char xrel, char yrel,
		     char xmach, char ymach, char xmem, char ymem)
{
  ANSI_screen_init ();

  draw_ansi_screen (screen, screen_length, screen_width);
  ANSI_save_pos ();



  ANSI_do_color (color1);
  ANSI_printxy (xnode, ynode, want_getty_infos ? (want_getty_infos == 1 ? "\\n" : "@S") : NODENAME);


  ANSI_do_color (color2);
  if (xtty + ytty)
    {
      if (isatty (0))
	{
	  ANSI_pos(xtty, ytty);
	  if (want_getty_infos)
	    ANSI_print (want_getty_infos == 1 ? "\\l" : "@L");
	  else
	    ANSI_print (ttyname(0) + 5);
	}
    }


  ANSI_do_color (color4);
  if ( WMESG && (xwmsg + ywmsg) )
    {
      ANSI_printxy (xwmsg, ywmsg, WMESG);
    }

  if (xtime + ytime)
    {
      ANSI_do_color (color2);
      ANSI_printxy (xtime, ytime, want_getty_infos ? (want_getty_infos == 1 ? "\\d" : "@D") : TIME);
    }

  if (MEMORY && (xmem + ymem))
    ANSI_printxy (xmem, ymem, MEMORY);

  if (CPUID && (xmach + ymach))
    {
      ANSI_do_color (color3);
      ANSI_printxy (xmach, ymach, CPUID);
    }

  if (Mhz && (xrel + yrel))
    {
      ANSI_do_color (color3);
      ANSI_printxy (xrel, yrel, Mhz);
    }

}



static void
draw_this_screen (void)
{
  int last_line = 1;

  switch (THIS_SCREEN)
    {
    default:
    case 1:
      {
#if 0
	char buf[80];
#endif
	draw_welcome_screen (WELCOME1, WELCOME1_LENGTH, WELCOME1_WIDTH,
			     0x07, 0x07, 0x07, 0x0b,
			     14, 11, 14, 12, 14, 13,
			     7, 15, 13, 18, 13, 17, 13, 16);
#if 0
	ANSI_do_color (0x19);
	sprintf (buf, "%s V%s (c)LittleIgloo.Org", PROG_NAME, PROG_VERSION);
	ANSI_printxy (0, last_line = 22, buf);
#endif
      }
      break;

    case 2:
      {
#if 0
	char buf[80];
#endif
	draw_welcome_screen (WELCOME2, WELCOME2_LENGTH, WELCOME2_WIDTH,
			     0x07, 0x07, 0x07, 0x0b,
			     14, 11, 14, 12, 14, 13,
			     7, 15, 13, 18, 13, 17, 13, 16);
#if 0
	ANSI_do_color (0x19);
	sprintf (buf, "%s %s (c)LittleIgloo.Org", PROG_NAME, PROG_VERSION);
	ANSI_printxy (0, last_line = 22, buf);
#endif
      }
      break;


    case 3:
      draw_welcome_screen (XMAS, XMAS_LENGTH, XMAS_WIDTH,
			   0x04, 0x04, 0x04, 0x0b,
			   15, 12, 0, 0, 0, 0,
			   0, 0, 24, 14, 22, 15, 23, 16);
      ANSI_do_color (0x04);
      ANSI_printxy (18, 18, "We wish you a");

      if (!NEW_YEAR)
	{
	  ANSI_do_color (0x8F);
	  ANSI_printxy (16, 19, "MERRY CHRISTMAS");
	}
      else
	{
	  char what_year[50];
	  sprintf (what_year, "HAPPY NEW YEAR %i", NEW_YEAR);
	  ANSI_do_char ('n');
	  ANSI_do_color (0x8F);
	  ANSI_printxy (13, 19, what_year);
	}
      last_line = 23;
      break;
    }


  if (!IS_WELCOME)
    {
      ANSI_do_color (0x4E);
      ANSI_printxy (0, last_line, " *** Unauthorized access is PROHIBITED ** Unauthorized access is PROHIBITED *** ");
    }


  ANSI_restore_pos ();
  ANSI_screen_close ();

  printf (WSCREEN);
}

/* Name and version of program.  */
static void print_version (FILE *stream, struct argp_state *state);
void (*argp_program_version_hook) (FILE *, struct argp_state *) = print_version;

/* Definitions of arguments for argp functions.  */
static const struct argp_option options[] =
{
  { NULL, 0, NULL, 0, "Output control:" },
  { "getty", 'g', NULL, 0, "display more accurate information for mingetty" },
  { "gettyps", 'G', NULL, 0, "display more accurate information for gettyps" },
  { "nolf", 'n', NULL, 0, "don't add an empty line to the ANSI screen produced" },
  { "msg", 't', "TEXT", 0, "show the TEXT given as the distribution info" },
  { "cpu", 'c', "NAME", 0, "show the NAME given as the CPU name" },
  { "memory", 'm', "SIZE", 0, "show the SIZE given as the memory size" },
  { "src", 's', "XX", 0, "display screen number XX" },
  { "random", 'r', NULL, 0, "display a random screen" },
  { "lcol", 'l', "XX", 0, "change the prompt color to the color number XX" },
  { "private", 'p', NULL, 0, "display an \"Unauthorized access is prohibited\" line at the bottom of the ANSI screen" },
  { NULL, 0, NULL, 0, "Color numbers: \033[0;34m1 \033[32m2 \033[36m3 \033[31m4 \033[35m5 \033[33m6 \033[37m7 \033[1;30m8 \033[34m9 \033[32m10 \033[36m11 \033[31m12 \033[35m13 \033[33m14 \033[37m15\033[m" },
  { NULL, 0, NULL, 0, "Example: " PROG_NAME " --getty --msg=\"ALT Linux\" --lcol=14" },
  { NULL, 0, NULL, 0, NULL }
};

/* Short description of program.  */
static const char doc[] = "Produce a BBS like ANSI login logo.";

/* Strings for arguments in help texts.  */
static const char args_doc[] = "";

/* Prototype for option handler.  */
static error_t parse_opt (int key, char *arg, struct argp_state *state);

/* Function to print some extra text in the help message.  */
static char *more_help (int key, const char *text, void *input);

/* Data structure to communicate with argp functions.  */
static struct argp argp =
{
  options, parse_opt, args_doc, doc, NULL, more_help
};

/* Handle program arguments.  */
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  switch (key)
    {
    case 'g':
      want_getty_infos = 1;
      break;
    case 'G':
      want_getty_infos = 2;
      break;
    case 'n':
      WANT_NEWLINE = FALSE;
      break;
    case 't':
      WMESG = arg;
      break;
    case 'c':
      CPUID = arg;
      break;
    case 'm':
      MEMORY = arg;
      break;
    case 's':
      THIS_SCREEN = atoi (arg);
      if (THIS_SCREEN < 1 || THIS_SCREEN > MAX_RAND_SCREEN)
	 THIS_SCREEN = 1;
      break;
    case 'r':
      {
	struct timeval tv;
	gettimeofday (&tv, 0);
	srandom (getpid() + tv.tv_usec);
	THIS_SCREEN = 1 + ((double)MAX_RAND_SCREEN * random())/RAND_MAX;
	break;
      }
    case 'x':
      THIS_SCREEN = 3;
      break;
    case 'X':
      XMAS_AUTO = TRUE;
      break;
    case 'l':
      last_color = atoi (arg);
      if ((last_color < 1) || (last_color > 15))
	 last_color = 15;
      break;
    case 'p':
      IS_WELCOME = FALSE;
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

static char *
more_help (int key, const char *text, void *input)
{
  switch (key)
    {
    case ARGP_KEY_HELP_EXTRA:
      /* We print some extra information.  */
      return strdup ("Report bugs to http://bugs.altlinux.ru/\n" );
    default:
      break;
    }
  return (char *) text;
}

/* Print the version information.  */
static void
print_version (FILE *stream, struct argp_state *state)
{
  fprintf (stream, "%s V%s By %s\nCopyright (c)%s %s\nCompiled on %s by %s@%s (%s)\n%s\n",
	PROG_NAME, PROG_VERSION, PROG_AUTHOR,
	VERSION_DATE, LITTLEIGLOO,
	COMPILE_DATE, COMPILE_BY, COMPILE_HOST, CC_VERSION,
	"Distributed under the terms of the GNU General Public License.");
}

static int
get_ram(void)
{
	struct stat stb;
	if ( stat ("/proc/kcore", &stb) < 0)
		return 0;
	else
		return ((stb.st_size >> 10) + 1023) >> 10;
}

static char *
get_cpu(void)
{
	char *cpu = 0;
	FILE *fp = fopen ("/proc/cpuinfo", "r");
	if (fp)
	{
		const char prefix[] = "model name";
		char line[4096];
		while (fgets(line, sizeof(line), fp))
		{
			if (!strncmp (line, prefix, sizeof(prefix) - 1))
			{
				const char *p = strchr (line + sizeof(prefix) - 1, ':');
				if (p)
				{
					for (++p; *p; ++p)
						if (!isspace(*p))
							break;
					cpu = strdup (p);
					break;
				}
			}
		}
		fclose (fp);
	}
	return cpu;
}

static int
get_mhz(void)
{
	int mhz = 0;
	FILE *fp = fopen ("/proc/cpuinfo", "r");
	if (fp)
	{
		const char prefix[] = "cpu MHz";
		char line[4096];
		while (fgets(line, sizeof(line), fp))
		{
			if (!strncmp (line, prefix, sizeof(prefix) - 1))
			{
				const char *p = strchr (line + sizeof(prefix) - 1, ':');
				if (p)
				{
					for (++p; *p; ++p)
						if (!isspace(*p))
							break;
					mhz = atoi (p);
					break;
				}
			}
		}
		fclose (fp);
	}
	return mhz;
}

int
main (int argc, char *argv[])
{
  time_t timep;
  struct tm *timeptr;
  int remaining = 0;

  printf ("\033[m");
  /* Parse and process arguments.  */
  argp_parse (&argp, argc, argv, 0, &remaining, NULL);

#if 0
  setlocale (LC_ALL, "");
#endif

  uname (&uname_buf);

  timep = time (NULL);
  timeptr = localtime (&timep);
  strftime (TIME, sizeof (TIME), "%a %d %b %Y", timeptr);



  if ((timeptr->tm_mon < 2) || (timeptr->tm_mon == 11 && timeptr->tm_mday > 25))
    NEW_YEAR = timeptr->tm_year + 1900 + (timeptr->tm_mon == 11 && timeptr->tm_mday > 25);

  if ((XMAS_AUTO) && ((timeptr->tm_mon == 0) || (timeptr->tm_mon == 11)))
    THIS_SCREEN = 3;		/* Ross Younger's patch (revised) - no Xmas decorations outside of Jan/Dec. (enable it with -xmasauto) */


  if ( !MEMORY )
    asprintf (&MEMORY, "%d Mb", get_ram());

  if ( !CPUID )
    CPUID = get_cpu();

  if ( !CPUID )
    CPUID = MACHINE;

  if ( !Mhz )
    asprintf (&Mhz, "%d", get_mhz());

  draw_this_screen ();

  if (last_color)
    {
      char attr = 0;
      if (last_color > 8)
	{
	  attr++;
	  last_color -= 8;
	}
      printf ("\033[%i;3%im", attr, ANSI_col_convert (last_color));
    }

  if (WANT_NEWLINE)
    printf ("\n");

  return 0;
}
